#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""About dialog."""

from PyQt5.QtGui import QIcon, QPixmap, QDesktopServices

from PyQt5.QtCore import Qt, QUrl
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QVBoxLayout, QLabel, QSizePolicy, QDialog, QPushButton, QTextEdit

from app.plugins.core.plugin import IdePlugin

HEADSPIN_URL = "https://headspin.io/trial"


class AboutDialog(QDialog):
    """About dialog."""

    def __init__(self, parent=None, title=""):
        super(AboutDialog, self).__init__(parent)

        self.window_title = title or "About Dialog"
        self.parent = parent

        self._initWidgetTexts()
        self.setupUi()
        self.connectSlots()

        self.setProperty("dialog_name", "about_dialog")
        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)

        self.resize(600, 450)

    def connectSlots(self):
        """连接信号-槽."""
        self.btn_box.clicked.connect(self.onClickOk)

    def getHeadspinPixmap(self):
        """获取HeadSpin的图标."""
        return QPixmap(":/images/firebase_window/headspin_logo_orange.svg")

    def _initWidgetTexts(self):
        """根据中英文显示内容."""
        locale = IdePlugin.conf.get("preferLocale", None)
        if locale == "zh_CN":
            self.window_title = "关于 HeadSpin"
            self.headspin_text = "    Headspin是Airtest Project项目组的金牌合作伙伴。网易和Headspin的合作将帮助App/游戏开发者面向全球用户开发高质量、高性能的App/游戏。\n\n    Headspin在全球150个国家拥有22, 000台远程真机设备，并即将支持Airtest Project测试框架。未来Airtest Project的用户将可以通过AirtestIDE直接使用来自Headspin的不同国家的设备；也可以直接使用Headspin的服务来运行测试脚本。"
            self.headspin_str = " 官网: <a href='%s' style='color:#69aeff;'>%s</a>" % (HEADSPIN_URL, HEADSPIN_URL)
            self.ok_btn_str = "    确定    "
        else:
            self.window_title = "About HeadSpin"
            self.headspin_text = "    HeadSpin is the gold partner of the Airtest Project team. Netease and Headspin partnership helps App/Game developers create high quality and high performing apps/games that delight users worldwide.\n\n    HeadSpin has 22,000 real devices in 150 countries around the world and will support the Airtest Project test framework soon. In the near future, Airtest Project users will be able to use devices of Headspin directly through AirtestIDE. Users can also run test scripts directly using HeadSpin's services."
            self.headspin_str = " Homepage: <a href='%s' style='color:#69aeff;'>%s</a>" % (HEADSPIN_URL, HEADSPIN_URL)
            self.ok_btn_str = "    OK    "

    def setupUi(self):
        """初始化界面."""
        # title + icon
        self.setWindowTitle(self.window_title)
        if not self.parent:
            self.setWindowIcon(QIcon(":/images/mainwindow/airtest_ide.png"))
        # 窗口内容：
        headspin_pixmap = self.getHeadspinPixmap()
        headspin_pixmap.scaled(259, 89, Qt.KeepAspectRatio)
        self.image_label = CustomLabel()
        self.image_label.setPixmap(headspin_pixmap)

        self.headspin_info_edit = QTextEdit()
        self.headspin_info_edit.setReadOnly(True)
        self.headspin_info_edit.setProperty("textedit_name", "about_info")
        self.headspin_info_edit.setText(self.headspin_text)

        self.headspin_label = QLabel(self.headspin_str)
        self.headspin_label.setOpenExternalLinks(True)
        self.btn_box = QPushButton(self.ok_btn_str)

        # 组装上部Ui
        top_widget = QWidget()
        top_widget.setMinimumHeight(130)
        self.top_layout = QHBoxLayout(top_widget)
        self.top_layout.setSpacing(4)
        self.top_layout.setContentsMargins(20, 20, 20, 20)
        top_widget.setProperty("widget_name", "about_info_widget")
        self.top_layout.addWidget(self.image_label, 0, Qt.AlignHCenter)

        # 组装下部Ui
        bottom_widget = QWidget()
        self.bottom_layout = QVBoxLayout(bottom_widget)
        self.bottom_layout.setSpacing(4)
        self.bottom_layout.setContentsMargins(20, 10, 20, 10)
        self.bottom_layout.addWidget(self.headspin_info_edit)
        self.bottom_layout.addWidget(self.headspin_label)

        # 组装ok-btn:
        ok_btn_box_widget = QWidget()
        self.ok_btn_layout = QHBoxLayout(ok_btn_box_widget)
        # self.ok_btn_layout.setSpacing(20)
        self.ok_btn_layout.setContentsMargins(20, 0, 20, 10)  # left top right bottom
        self.ok_btn_layout.addWidget(self.btn_box, 0, Qt.AlignRight)

        self.main_layout = QVBoxLayout(self)
        self.main_layout.setSpacing(4)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.addWidget(top_widget)
        self.main_layout.addWidget(bottom_widget)
        self.main_layout.addWidget(ok_btn_box_widget)

    def onClickOk(self):
        """点击确认按钮."""
        self.close()


class CustomLabel(QLabel):
    """Custom label with mouse press slot."""

    def mousePressEvent(self, event):
        super(CustomLabel, self).mousePressEvent(event)
        QDesktopServices.openUrl(QUrl(HEADSPIN_URL))
